MODULE GraphicsDemo;

(* Dieses Demo veranschaulicht die Benutzung der Objekt-Routinen aus *)
(* dem Modul GraphicsSupport. Auf diesem Demo liegt kein Copyright!  *)

FROM SYSTEM          IMPORT ADR,SHORTSET,BITSET,FFP,ASSEMBLE,ADDRESS;
FROM GraphicsSupport IMPORT SnapSprite,MoveSpriteTo,SpriteHandlePtr,
                            SetSpriteColors,RemSprite,SetSpriteCollision,
                            GetSpriteCollision,SpriteCollisionSet,
                            SpriteCollisions,SaveSprite,LoadSprite,
                            GetVSprite,MoveVSprite,RedrawGels,GetGelsInfo,
                            FreeGelsInfo,SaveVSprite,LoadVSprite,FreeVSprite,
                            BobTypeFlagSet,BobTypeFlags,GetBob,FreeBob,
                            MoveBob,SetBorderCollisionProcedure,border,
                            CollisionBorderSet,SetGelsCollisionProcedure,
                            CollisionBorders,TestCollision,GetView,
                            FreeView,SetView,ViewHandlePtr,AllGelsInfoPtr,
                            InitAnim,AddAnimBob,EndAnimDefinition,AnimPtr,
                            SetAnim,GetActualVelocity,GetActualPosition,
                            AnimObjects,FreeAnim,ChangeBitMap,GetBitMap,
                            GetRastPort,WriteText,FreeRastPort,FreeBitMap,
                            SetBobImage,GetBobImage,RedrawWaitMode;
FROM MathTrans       IMPORT Sin;
FROM MathFFP         IMPORT Abs;
FROM IntuitionD      IMPORT IntuitionBasePtr;
FROM GraphicsD       IMPORT ViewModeSet,RastPortPtr,ViewPortPtr,VSpritePtr,
                            GelsInfoPtr,BobPtr,BitMapPtr,jam1,jam2,ViewModes;
FROM GraphicsL       IMPORT RectFill,SetAPen,SetRGB4,WaitTOF,SetDrMd;
FROM Hardware        IMPORT custom,CollisionFlags,CollisionFlagSet;
FROM FFPConversions  IMPORT RealToStr;
FROM RandomNumber    IMPORT RND;
IMPORT IntuitionL;


(*$ EntryExitCode:=FALSE *)
PROCEDURE Colors;
BEGIN
  ASSEMBLE(DC.W $0000,$0FFF,$0F00,$0F0F,$0FF0,$00FF,$00F0,$000F,
                $0F82,$02F8,$082F,$0F28,$0333,$0666,$0999,$0CCC,
                $0FF6,$037A,$09FC,$0ADD,$0449,$0E05,$050E,$005E,
                $00E5,$0E50,$05E0,$0F83,$038F,$08F2,$0F33,$05F5
  END);
END Colors;



VAR vh,vh2      : ViewHandlePtr;
    rp,rp2,rp3  : RastPortPtr;
    bm          : BitMapPtr;
    vp          : ViewPortPtr;
    s1,s2,s3    : SpriteHandlePtr;
    anim,list   : AnimPtr;
    set         : SpriteCollisionSet;
    cc          : CollisionFlagSet;
    gelsInfo    : AllGelsInfoPtr;
    gelsInfo2   : AllGelsInfoPtr;
    bob         : BobPtr;
    iBase       : IntuitionBasePtr;
    px,py,vx,vy : FFP;
    ax,ay       : FFP;
    t,i,q,x,y   : INTEGER;
    err,dx      : BOOLEAN;
    v           : ARRAY[0..19] OF VSpritePtr;
    bb          : ARRAY[0..7] OF BobPtr;
    str         : ARRAY[0..40] OF CHAR;
    xpos,d      : ARRAY[0..19] OF INTEGER;
    sin         : ARRAY[0..360] OF INTEGER;
    card        : POINTER TO ARRAY[0..32] OF CARDINAL;



PROCEDURE VToV(v1 : VSpritePtr;    (* GEL-GEL Kollisionsroutine *)
               v2 : VSpritePtr);
VAR v : VSpritePtr;
    c : POINTER TO CARDINAL;
BEGIN
  IF (v2#NIL) THEN
    c:=v2^.sprColors;    (* Farben des Opfers *)
    IF c#NIL THEN        (* Hintergrundfarbe neu setzen *)
      SetRGB4(vp,0,(c^ DIV 256) MOD 16,(c^ DIV 16) MOD 16,c^ MOD 16);
    END;
  END;
END VToV;



PROCEDURE VToBack(v : VSpritePtr;  (* GEL-Rand Kollisionsroutine *)
                  b : CollisionBorderSet);
BEGIN
  IF hitTop IN b THEN
    SetRGB4(vp,0,15,0,0);
  ELSIF hitBottom IN b THEN
    SetRGB4(vp,0,0,15,0);
  ELSIF hitLeft IN b THEN
    SetRGB4(vp,0,0,0,15);
  ELSIF hitRight IN b THEN
    SetRGB4(vp,0,15,15,0);
  ELSE
    SetRGB4(vp,0,15,15,15);
  END;
END VToBack;


BEGIN

  iBase:=ADR(IntuitionL);  (* fr die Abfrage der Mausposition *)
  vh:=GetView(320,256,4,ViewModeSet{sprites},ADR(Colors));

  IF vh#NIL THEN
    rp:=ADR(vh^.rastPort);
    vp:=ADR(vh^.viewPort);
    SetView(vh);
    FOR t:=15 TO 0 BY -1 DO
      SetAPen(rp,t);
      RectFill(rp,0,16,t*10,16+t*10); (* Rechtecke zeichnen *)
    END;

    (* ******************** 1. Demo : Hardwaresprites ******************* *)
    (* ****** und Kollisionen untereinander und mit dem Hintergrund ***** *)

    s1:=SnapSprite(4,rp,0,16, 60,0,0,TRUE);   (* Attached Sprite *)
    s2:=SnapSprite(2,rp,4,17, 18,0,0,FALSE);
    s3:=SnapSprite(6,rp,4,17, 18,0,0,FALSE);

    SetSpriteCollision(SHORTSET{3},SHORTSET{3},SHORTSET{0..7},511);
                    (* Kollisionen werden auf Biplane 3 des Hintergrundes *)
                    (* geprft, ob sich dort ein gesetztes Bit befindet.  *)
                    (* Es werden alle Sprites zur Kollisionsprfung heran-*)
                    (* gezogen ({0..7}). Es wird 512 mal geprft.         *)

    card:=vp^.colorMap^.colorTable;
    SetSpriteColors(s1,vp,card^);  (* SpriteFarben = Bildschirmfarben *)

    WriteText(rp,0,200, 1,"Sprite-Sprite und           ");
    WriteText(rp,0,208, 1,"Sprite-Background Collisions");

    FOR t:=0 TO 600 DO
      (* Sprites mit der Maus bewegen *)
      MoveSpriteTo(s1,vp,320-iBase^.mouseX/2,iBase^.mouseY/2);
      MoveSpriteTo(s2,vp,320-iBase^.mouseX/2,256-iBase^.mouseY/2);
      MoveSpriteTo(s3,vp,iBase^.mouseX/2,256-iBase^.mouseY/2);

      set:=GetSpriteCollision(s2); (* Kollisionen testen *)
      IF (sprite4or5 IN set) THEN  (* Attached Sprite *)
        SetRGB4(vp,0,0,0,15);
      ELSIF (sprite6or7 IN set) THEN (* Sprite s3 *)
        SetRGB4(vp,0,0,15,0);
      ELSIF (oddPlane IN set) OR (evenPlane IN set) THEN
        SetRGB4(vp,0,15,0,0);
      ELSE
        SetRGB4(vp,0,0,0,0);
      END;
    END;

    RemSprite(s1);  (* Sprites wieder entfernen + lschen *)
    RemSprite(s2);
    RemSprite(s3);

    (* ******** 2. Demo : kreisende VSprites und ein Bob ********* *)
    (* ******** und Kollisionsabfrage VSprites und Rand  ********* *)

    (* Es werden die Hardwaresprites 1-7 zur VSprite darstellung benutzt *)
    gelsInfo:=GetGelsInfo(rp,SHORTSET{1..7},0,300,0,226); (* Rand *)
    IF gelsInfo#NIL THEN

      FOR t:=0 TO 19 DO (* besorgt 20 VSprites *)
        v[t]:=GetVSprite(rp,1,17,8,TRUE,TRUE,BITSET{},BITSET{1},
                         ADR(card^[(t MOD 12)+1]),rp,100,100);
        d[t]:=(t*18 MOD 360);
      END;

      bob:=GetBob(rp,0,16,40,40,4,BobTypeFlagSet{saveBackground,transparent},
                  BITSET{1,border},BITSET{},NIL,rp,100,100);
                  (* Routine 1 und Hintergrund *)

      SetBorderCollisionProcedure(VToBack,gelsInfo); (* Randkollision *)
      SetGelsCollisionProcedure(1,VToV,gelsInfo);    (* GELkollision *)

      FOR t:=0 TO 359 DO (* legt eine Sinustabelle an *)
        sin[t]:=TRUNC(80.0*Sin(FFP(t)*3.14159/180.0));
      END;

      WriteText(rp,0,200, 1,"Bob-VSprite und              ");
      WriteText(rp,0,208, 1,"Bob-Rand Collisions          ");
      WriteText(rp,250,30, 1,"VSprites");

      FOR i:=0 TO 600 DO

        FOR t:=0 TO 19 DO
          q:=d[t]; (* VSprites kreisen lassen *)
          MoveVSprite(v[t],160+sin[(90-q) MOD 360],100+sin[q]);
          d[t]:=(q+2) MOD 360;
        END;

        MoveBob(bob,iBase^.mouseX/2-20,iBase^.mouseY/2-20);

        IF (600-i)/10<19 THEN
          FreeVSprite(v[(600-i)/10],rp,vp,ADR(vh^.view),FALSE);
        END;

        (* zeichnet alle Objekte neu *)
        RedrawGels(rp,vp,ADR(vh^.view),waitBOVP); (* Sorry *)

        TestCollision(rp); (* prft auf Kollisionen *)
      END;
      FreeBob(bob,rp,vp,ADR(vh^.view),FALSE);
    END;

    (* ************* 3. Demo : hpfendes Bob *************** *)
    (* ****** als Animation im Double Buffering Modus ****** *)

    vh2:=GetView(320,256,4,ViewModeSet{},ADR(Colors));
    IF vh2#NIL THEN
      rp2:=ADR(vh2^.rastPort);
      gelsInfo2:=GetGelsInfo(rp2,SHORTSET{},0,320,0,250);

      SetAPen(rp,0);
      RectFill(rp,55,55,75,75); (* Rechteck ausschneiden (transparent) *)

      anim:=InitAnim(); (* Animation vorbereiten *)

      FOR t:=0 TO 7 DO
        bb[t]:=GetBob(rp,t*4,25,100,100,4,
                      BobTypeFlagSet{saveBackground,transparent,
                                     doubleBuffering,animBob},
                      BITSET{},BITSET{},bb[t MOD 2],rp2,40,40);
        AddAnimBob(anim,bb[t],ABS(4-t)+1, 0.0,0.0);
      END;
      FOR t:=7 TO 0 BY -1 DO (* Fr diese 8 Bobs werden nur Strukturen, *)
                             (* wenn mglich im Fast-RAM, und die Mask- *)
                             (* buffer im Chip-RAM belegt!              *)
                             (* Mask nur ca. 1313 Bytes pro Bob!        *)
        bb[t+8]:=GetBob(NIL,0,0,100,100,4,
                        BobTypeFlagSet{saveBackground,transparent,
                                       doubleBuffering,animBob,noImage},
                        BITSET{},BITSET{},bb[(7-t) MOD 2],rp2,40,40);
        SetBobImage(bb[t+8],GetBobImage(bb[t]),FALSE);
        AddAnimBob(anim,bb[t+8],ABS(4-t)+1, 0.0,0.0);
      END;

      list:=NIL;
      EndAnimDefinition(list,anim,rp2,NIL);

      FreeView(vh);
      SetView(vh2);

      bm:=GetBitMap(320,256,4);
      IF bm#NIL THEN
        rp3:=GetRastPort(bm);

        SetDrMd(rp2,jam1); SetDrMd(rp3,jam1);
        FOR t:=0 TO 200 DO
          x:=RND(280); y:=RND(190)+16; i:=RND(15)+1;
          WriteText(rp3,x,y,i,"Demo");
          WriteText(rp2,x,y,i,"Demo");
        END;

        (* Auf beiden Bildschirmen das gleiche zeichnen *)
        SetDrMd(rp2,jam2); SetDrMd(rp3,jam2);
        WriteText(rp2,0,250, 1,"  Double Buffering-Animation von Bobs  ");
        WriteText(rp3,0,250, 1,"  Double Buffering-Animation von Bobs  ");

        WriteText(rp2,0,10, 1," X:"); WriteText(rp3,0,10, 1," X:");
        WriteText(rp2,0,18, 1," Y:"); WriteText(rp3,0,18, 1," Y:");
        WriteText(rp2,0,26, 1,"vX:"); WriteText(rp3,0,26, 1,"vX:");
        WriteText(rp2,0,34, 1,"vY:"); WriteText(rp3,0,34, 1,"vY:");

        ay:=8.1/64.0; ax:=2.1/64.0;
        SetAnim(anim,50.0, 50.0,  0.2, 0.0,  ax, ay);
        dx:=FALSE;

        FOR t:=0 TO 1600 DO
          AnimObjects(list); (* Animiert das Objekt (Bewegung) *)

          GetActualPosition(anim,px,py);
          GetActualVelocity(anim,vx,vy);
          IF ((px<0.0) AND dx) OR ((px>220.0) AND NOT dx) THEN
            vx:=-vx; dx:=NOT dx;
          END;
          IF (py<0.0) OR (py>144.0) THEN
            vy:=-(vy-vy/32.0);
          END;

          IF Abs(vx)>3.0 THEN (* Wenn die X-Geschwindigkeit 3 Pixel/Aufruf *)
            ax:=0.0;          (* berschreitet wird die Beschleunigung     *)
          END;                (* auf 0 heruntergesetzt.                    *)

          SetAnim(anim,px,py, vx,vy, ax,ay); (* Daten bertragen *)

          IF t>1000 THEN (* Bewegungsdaten anzeigen *)
            RealToStr(px,str,-5,2,FALSE,err); WriteText(rp2,24,10,1,str);
            RealToStr(py,str,-5,2,FALSE,err); WriteText(rp2,24,18,1,str);
            RealToStr(vx,str,-5,2,FALSE,err); WriteText(rp2,24,26,1,str);
            RealToStr(vy,str,-5,2,FALSE,err); WriteText(rp2,24,34,1,str);
          END;

            (* 'waitTOF' wird nicht unbedingt bentigt. Ohne 'waitTOF'  *)
            (* flimmert die Ausgabe zwar nicht (dank Double Buffering), *)
            (* allerdings kann die Bewegung etwas ruckelig werden.      *)
          RedrawGels(rp2,ADR(vh2^.viewPort),ADR(vh2^.view),waitTOF);

          bm:=ChangeBitMap(rp2,ADR(vh2^.viewPort),bm); (* Double Buffering *)
                                                       (* umschalten       *)

        END;
        FOR t:=0 TO 7 DO (* Bobs wieder freigeben *)
          FreeBob(bb[t],rp2,ADR(vh2^.viewPort),ADR(vh2^.view),FALSE);
        END;
      END;
    END;
  END;

CLOSE

  FOR t:=0 TO 7 DO
    FreeBob(bb[t],NIL,NIL,NIL,FALSE);
  END;
  FreeBob(bob,NIL,NIL,NIL,FALSE);
  FreeAnim(anim);
  FreeView(vh);
  FreeView(vh2);
  FreeRastPort(rp3);
  FreeBitMap(bm);
  FreeGelsInfo(gelsInfo);
  FreeGelsInfo(gelsInfo2);

END GraphicsDemo.
